@@ -61,9 +61,6 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
                                   GtkStateType          state_type,
                                   WidgetParameters     *params)
 {
-	if (widget && GE_IS_ENTRY (widget))
-		state_type = GTK_WIDGET_STATE (widget);
-
 	params->style_functions = &(clearlooks_style_class->style_functions[CLEARLOOKS_STYLE (style)->style]);
 
 	params->active      = (state_type == GTK_STATE_ACTIVE);
@@ -168,6 +165,11 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
+		/* Override the entries state type, because we are too lame to handle this via
+		 * the focus ring, and GtkEntry doesn't even set the INSENSITIVE state ... */
+		if (state_type == GTK_STATE_NORMAL && widget && GE_IS_ENTRY (widget))
+			params.state_type = GTK_WIDGET_STATE (widget);
+
 		if (widget && (ge_is_in_combo_box (widget) || GE_IS_SPIN_BUTTON (widget)))
 		{
 			width += style->xthickness;
@@ -254,6 +256,7 @@ clearlooks_style_draw_box_gap (DRAW_ARGS,
 	{
 		WidgetParameters params;
 		FrameParameters  frame;
+		gboolean start, end;
 		
 		frame.shadow    = shadow_type;
 		frame.gap_side  = gap_side;
@@ -263,24 +266,47 @@ clearlooks_style_draw_box_gap (DRAW_ARGS,
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
+		clearlooks_get_notebook_tab_position (widget, &start, &end);
+
+		params.corners = CR_CORNER_ALL;
 		switch (gap_side) {
 			case GTK_POS_LEFT:
-				params.corners = CR_CORNER_ALL ^ CR_CORNER_TOPLEFT;
+				if (start)
+					params.corners ^= CR_CORNER_TOPLEFT;
+				if (end)
+					params.corners ^= CR_CORNER_BOTTOMLEFT;
 			break;
 			case GTK_POS_RIGHT:
-				params.corners = CR_CORNER_ALL ^ CR_CORNER_TOPRIGHT;
+				if (start)
+					params.corners ^= CR_CORNER_TOPRIGHT;
+				if (end)
+					params.corners ^= CR_CORNER_BOTTOMRIGHT;
 			break;
 			case GTK_POS_TOP:
-				if (ge_widget_is_ltr (widget))
-					params.corners = CR_CORNER_ALL ^ CR_CORNER_TOPLEFT;
-				else
-					params.corners = CR_CORNER_ALL ^ CR_CORNER_TOPRIGHT;
+				if (ge_widget_is_ltr (widget)) {
+					if (start)
+						params.corners ^= CR_CORNER_TOPLEFT;
+					if (end)
+						params.corners ^= CR_CORNER_TOPRIGHT;
+				} else {
+					if (start)
+						params.corners ^= CR_CORNER_TOPRIGHT;
+					if (end)
+						params.corners ^= CR_CORNER_TOPLEFT;
+				}
 			break;
 			case GTK_POS_BOTTOM:
-				if (ge_widget_is_ltr (widget))
-					params.corners = CR_CORNER_ALL ^ CR_CORNER_BOTTOMLEFT;
-				else
-					params.corners = CR_CORNER_ALL ^ CR_CORNER_BOTTOMRIGHT;
+				if (ge_widget_is_ltr (widget)) {
+					if (start)
+						params.corners ^= CR_CORNER_BOTTOMLEFT;
+					if (end)
+						params.corners ^= CR_CORNER_BOTTOMRIGHT;
+				} else {
+					if (start)
+						params.corners ^= CR_CORNER_BOTTOMRIGHT;
+					if (end)
+						params.corners ^= CR_CORNER_BOTTOMLEFT;
+				}
 			break;
 		}
 
@@ -794,7 +820,11 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		scrollbar.has_color  = FALSE;
 		scrollbar.horizontal = TRUE;
 		scrollbar.junction   = clearlooks_scrollbar_get_junction (widget);
-		
+
+		if (clearlooks_style->colorize_scrollbar || clearlooks_style->has_scrollbar_color) {
+			scrollbar.has_color = TRUE;
+		}
+
 		scrollbar.horizontal = DETAIL ("hscrollbar");
 		
 		stepper.stepper = clearlooks_scrollbar_get_stepper (widget, &this_rectangle);
@@ -828,6 +858,24 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		
 		STYLE_FUNCTION(draw_menu_frame) (cr, colors, &params, x, y, width, height);
 	}
+	else if (DETAIL ("hseparator") || DETAIL ("vseparator"))
+	{
+		gchar *new_detail = detail;
+		/* Draw a normal separator, we just use this because it gives more control
+		 * over sizing (currently). */
+
+		/* This isn't nice ... but it seems like the best cleanest way to me right now.
+		 * It will get slightly nicer in the future hopefully. */
+		if (GE_IS_MENU_ITEM (widget))
+			new_detail = "menuitem";
+
+		if (DETAIL ("hseparator")) {
+			gtk_paint_hline (style, window, state_type, area, widget, new_detail,
+			                 x, x + width - 1, y + height/2);
+		} else
+			gtk_paint_vline (style, window, state_type, area, widget, new_detail,
+			                 y, y + height - 1, x + width/2);
+	}
 	else
 	{
 		clearlooks_parent_class->draw_box (style, window, state_type, shadow_type, area,
@@ -962,15 +1010,15 @@ clearlooks_style_draw_check (DRAW_ARGS)
 }
 
 static void
-clearlooks_style_draw_vline                      (GtkStyle               *style,
-                                 GdkWindow              *window,
-                                 GtkStateType            state_type,
-                                 GdkRectangle           *area,
-                                 GtkWidget              *widget,
-                                 const gchar            *detail,
-                                 gint                    y1,
-                                 gint                    y2,
-                                 gint                    x)
+clearlooks_style_draw_vline (GtkStyle               *style,
+                             GdkWindow              *window,
+                             GtkStateType            state_type,
+                             GdkRectangle           *area,
+                             GtkWidget              *widget,
+                             const gchar            *detail,
+                             gint                    y1,
+                             gint                    y2,
+                             gint                    x)
 {
 	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
 	const ClearlooksColors *colors;
@@ -982,23 +1030,25 @@ clearlooks_style_draw_vline                      (GtkStyle               *style,
 	colors = &clearlooks_style->colors;
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
-	
-	STYLE_FUNCTION(draw_separator) (cr, &colors->shade[3], NULL, &separator,
-	                           x, y1, 2, y2-y1);
+
+	/* There is no such thing as a vertical menu separator
+	 * (and even if, a normal one should be better on menu bars) */
+	STYLE_FUNCTION(draw_separator) (cr, colors, NULL, &separator,
+	                                x, y1, 2, y2-y1+1);
 	
 	cairo_destroy (cr);
 }
 
 static void
-clearlooks_style_draw_hline                      (GtkStyle               *style,
-                                 GdkWindow              *window,
-                                 GtkStateType            state_type,
-                                 GdkRectangle           *area,
-                                 GtkWidget              *widget,
-                                 const gchar            *detail,
-                                 gint                    x1,
-                                 gint                    x2,
-                                 gint                    y)
+clearlooks_style_draw_hline (GtkStyle               *style,
+                             GdkWindow              *window,
+                             GtkStateType            state_type,
+                             GdkRectangle           *area,
+                             GtkWidget              *widget,
+                             const gchar            *detail,
+                             gint                    x1,
+                             gint                    x2,
+                             gint                    y)
 {
 	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
 	const ClearlooksColors *colors;
@@ -1013,8 +1063,12 @@ clearlooks_style_draw_hline                      (GtkStyle               *style,
 	
 	separator.horizontal = TRUE;
 	
-	STYLE_FUNCTION(draw_separator) (cr, &colors->shade[3], NULL, &separator,
-	                           x1, y, x2-x1, 2);
+	if (!DETAIL ("menuitem"))
+		STYLE_FUNCTION(draw_separator) (cr, colors, NULL, &separator,
+		                                x1, y, x2-x1+1, 2);
+	else
+		STYLE_FUNCTION(draw_menu_item_separator) (cr, colors, NULL, &separator,
+		                                           x1, y, x2-x1+1, 2);
 	
 	cairo_destroy (cr);
 }
